﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using LocalSearch;
using Microsoft.Msagl.Drawing;
using Color = Microsoft.Msagl.Drawing.Color;

namespace FitnessLandscapeTopology
{
    public partial class Form1 : Form
    {
        Microsoft.Msagl.GraphViewerGdi.GViewer drawer;
        Graph g;
        int size;
        Func<List<int>, double> evaluator;

        public Form1(int size, Func<List<int>, double> evaluator)
        {
            InitializeComponent();
            drawer = new Microsoft.Msagl.GraphViewerGdi.GViewer();
            drawer.Dock = DockStyle.Fill;
            right_panel.Controls.Add(drawer);
            this.size = size;
            this.evaluator = evaluator;

            g = CreateGraph(size, Topology.linear, evaluator);
            drawer.Graph = g;
            drawer.CurrentLayoutMethod = Microsoft.Msagl.GraphViewerGdi.LayoutMethod.MDS;
            drawer.Refresh();
        }

        public Form1()
        {
            InitializeComponent();
            drawer = new Microsoft.Msagl.GraphViewerGdi.GViewer();
            drawer.Dock = DockStyle.Fill;
            right_panel.Controls.Add(drawer);
        }

        private void draw_button_Click(object sender, EventArgs e)
        {
            if (evaluator != null)
                g = CreateGraph(this.size, ParseTopology(), this.evaluator);
            else
                g = CreateGraph(int.Parse(textBox1.Text), ParseTopology());
            drawer.Graph = g;
            drawer.CurrentLayoutMethod = Microsoft.Msagl.GraphViewerGdi.LayoutMethod.MDS;
            drawer.Refresh();
        }

        private Topology ParseTopology()
        {
            if (listBox1.SelectedItem.ToString() == "Linear") return Topology.linear;
            if (listBox1.SelectedItem.ToString() == "SingleSwap") return Topology.swap;
            if (listBox1.SelectedItem.ToString() == "3-rotation") return Topology.rotation;
            throw new NotImplementedException();
        }

        private Graph CreateGraph(int size, Topology topology, Func<List<int>, double> evaluator = null)
        {
            Dictionary<string, double> evaluations = new Dictionary<string, double>();
            Dictionary<string, Node> nodes = new Dictionary<string, Node>();

            double minVal = double.MaxValue, maxVal = double.MinValue;

            Graph g = new Graph();
            foreach (var item in GenerateAllTSPSolutions(size))
            {
                string perm = string.Join(" ", item);
                var node = g.AddNode(perm);
                nodes.Add(perm, node);
                if (evaluator != null)
                {
                    var eval = evaluator.Invoke(item);
                    evaluations.Add(perm, eval);

                    if (eval < minVal)
                        minVal = eval;
                    if (eval > maxVal)
                        maxVal = eval;
                }
            }
            if (evaluator != null)
            {
                foreach (var item in GenerateAllTSPSolutions(size))
                {
                    var val = evaluations[string.Join(" ", item)];
                    var color = ShiftColor(Color.Green, Color.Yellow, Color.Red, minVal, (minVal + maxVal) / 2, maxVal, val);
                    var node = nodes[string.Join(" ", item)];
                    node.Attr.FillColor = color;
                    node.Attr.Color = color;
                }
            }

            switch (topology)
            {
                case Topology.linear:
                    var nodes1 = g.Nodes.OrderBy(q => q.Id).ToList();
                    for (int i = 0; i < nodes1.Count - 1; i++)
                    {
                        var edge = g.AddEdge(nodes1[i].Id, nodes1[i + 1].Id);
                        edge.Attr.Color = Color.LightGray;
                    }
                    break;
                case Topology.swap:
                    foreach (var item in GenerateAllTSPSolutions(size))
                    {
                        for (int i = 0; i < item.Count; i++)
                        {
                            for (int j = i + 1; j < item.Count; j++)
                            {
                                var newData = item.ToArray();
                                int temp = newData[i];
                                newData[i] = newData[j];
                                newData[j] = temp;

                                var node1 = string.Join(" ", item);
                                var node2 = string.Join(" ", newData);
                                if (node1.CompareTo(node2) < 0)
                                {
                                    if (nodes.ContainsKey(node1) && nodes.ContainsKey(node2))
                                    {
                                        var edge = g.AddEdge(node1, node2);
                                        edge.Attr.Color = Color.LightGray;
                                    }
                                }
                            }
                        }
                    }
                    break;
                case Topology.rotation:
                    foreach (var item in GenerateAllTSPSolutions(size))
                    {
                        for (int i = 0; i < item.Count; i++)
                        {
                            for (int j = i + 1; j < item.Count; j++)
                            {
                                for (int k = 0; k < j + 1; k++)
                                {
                                    var newData = item.ToArray();

                                    int temp = newData[i];
                                    newData[i] = newData[j];
                                    newData[j] = newData[k];
                                    newData[k] = temp;

                                    var node1 = string.Join(" ", item);
                                    var node2 = string.Join(" ", newData);
                                    if (node1.CompareTo(node2) < 0)
                                    {
                                        if (nodes.ContainsKey(node1) && nodes.ContainsKey(node2))
                                        {
                                            var edge = g.AddEdge(node1, node2);
                                            edge.Attr.Color = Color.LightGray;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    break;
                default:
                    break;
            }

            return g;
        }

        private IEnumerable<List<int>> GenerateAllTSPSolutions(int nodesCount)
        {
            HashSet<string> res = new HashSet<string>();
            foreach (var item in Combinatorics.GetAllPermutations(Enumerable.Range(1, size - 1).ToList()))
            {
                var str = string.Join(" ", item);
                item.Reverse();
                var strReversed = string.Join(" ", item);
                if (strReversed.CompareTo(str) > 0)
                {
                    if (!res.Contains(str) && !res.Contains(strReversed))
                    {
                        res.Add(str);
                        res.Add(strReversed);
                        item.Reverse();
                        item.Insert(0, 0);
                        yield return item;
                    }
                }
            }    
        }

        private static Color ShiftColor(Color minC, Color neutralC, Color maxC, double minVal, double neutralVal, double maxVal, double currentVal)
        {
            if (currentVal < neutralVal)
                return ShiftColor(minC, neutralC, minVal, neutralVal, currentVal);
            if (currentVal == neutralVal)
                return neutralC;
            else return ShiftColor(neutralC, maxC, neutralVal, maxVal, currentVal);
        }

        private static Color ShiftColor(Color minColor, Color maxColor, double minVal, double maxVal, double currentVal)
        {
            if (minVal >= maxVal)
                return minColor;

            if (currentVal < minVal)
                currentVal = minVal;
            if (currentVal > maxVal)
                currentVal = maxVal;

            double firstWeigth = (maxVal - currentVal) / (maxVal - minVal);
            double secondWeigth = (currentVal - minVal) / (maxVal - minVal);
            Color intermediate = new Color((byte)(minColor.R * firstWeigth + maxColor.R * secondWeigth),
                (byte)(minColor.G * firstWeigth + maxColor.G * secondWeigth),
                (byte)(minColor.B * firstWeigth + maxColor.B * secondWeigth));
            return intermediate;
        }
    }

    enum Topology
    {
        linear, swap, rotation
    }
}
