﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LocalSearch
{
    public class HillClimbingSolver<T, P> where T : CandidateSolution<P> where P : LocalSearchProblem
    {
        private Random r;
        private T current;
        private bool stop;
        private int steps, stepsWithoutImprovement;
        private T bestSoFar;
        private double currentBestVal;

        private LSNullaryOperator<T, P> initializationOperator;
        private LSUnaryOperator<T, P> modificationOperator;

        public T Solve(P input, Action<T, int> onImprovementCallback = null)
        {
            reset();
            current = initialize(input);
            bestSoFar = current;
            currentBestVal = current.Evaluate();
            
            while (!stop)
            {
                steps++;
                current = DoLocalStep(current);
                var currentVal = current.Evaluate();
                if (currentVal < currentBestVal)
                {
                    onImprovementCallback?.Invoke(current, steps);
                    currentBestVal = currentVal;
                    bestSoFar = current;
                }
            }
            return bestSoFar;
        }

        /// <summary>
        /// Tato metoda by mela vylepsit reseni ulozene v promenne <paramref name="currentSolution"/>. Kvalitu reseni je mozne ziskat volanim "currentSolution.Evaluate()". 
        /// Pokud uz soucasne reseni nelze vylepsit, metoda by mela nastavit promenou "stop" na "true", coz zpusobi, ze prohledavani skonci.
        /// Pro vylepseni (nebo obecne zmenu) reseni pouzijte nekterou variantu Hill-Climbingu nebo Simulovanoho zihani.
        /// </summary>
        protected virtual T DoLocalStep(T currentSolution)
        {
            //TODO

            stop = true;
            return currentSolution;
        }

        private T initialize(P input)
        {
            return initializationOperator.Apply(input);
        }

        private void reset()
        {
            stop = false;
            steps = 0;
            stepsWithoutImprovement = 0;
            currentBestVal = double.MaxValue;
        }

        public HillClimbingSolver(Random r, LSNullaryOperator<T, P> initializationOperator, LSUnaryOperator<T, P> modificationOperator)
        {
            this.r = r;
            this.initializationOperator = initializationOperator;
            this.modificationOperator = modificationOperator;
        }

    }
}
