﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TSP
{
    class CentroidBubbleSolver : TSPSolver
    {
        private double centroidX, centroidY;
        private double[] angles;
        Comparer<int> comparer;

        private void computeCentroid(TSPInput input)
        {
            angles = new double[input.NodesCount];
            double sumX = 0, sumY = 0;
            for (int i = 0; i < input.NodesCount; i++)
            {
                sumX += input.GetPoint(i).x;
                sumY += input.GetPoint(i).y;
            }
            centroidX = sumX / input.NodesCount;
            centroidY = sumY / input.NodesCount;
            for (int i = 0; i < input.NodesCount; i++)
            {
                angles[i] = Math.Atan2(input.GetPoint(i).x - centroidX, input.GetPoint(i).y - centroidY);
            }
            this.comparer = new referenceComparer(angles);
        }

        #region TSPSolver Members

        public TSPSolution Solve(TSPInput input)
        {
            computeCentroid(input);
            int[] points = new int[input.NodesCount];
            for (int i = 0; i < input.NodesCount; i++)
                points[i] = i;
            Array.Sort(points, comparer);
            TSPSolution sol = new TSPSolution(input);
            for (int i = 0; i < input.NodesCount - 1; i++)
            {
                sol.SetSuccessor(points[i], points[i + 1]);
            }
            sol.SetSuccessor(points[input.NodesCount - 1], points[0]);
            return sol;
        }

        #endregion

        private class centroidAngleComparer : Comparer<int>
        {
            private double centroidX, centroidY;
            private TSPInput inp;

            private double getDistanceSquared(int pointIndex)
            {
                return (inp.GetPoint(pointIndex).x - centroidX) * (inp.GetPoint(pointIndex).x - centroidX) +
                    (inp.GetPoint(pointIndex).y - centroidY) * (inp.GetPoint(pointIndex).y - centroidY);
            }

            private double getAngle(int pointIndex)
            {
                return Math.Atan2(inp.GetPoint(pointIndex).x - centroidX, inp.GetPoint(pointIndex).y - centroidY);
            }

            public centroidAngleComparer(double centroidX, double centroidY, TSPInput inp)
            {
                this.inp = inp;
                this.centroidX = centroidX;
                this.centroidY = centroidY;
            }

            public override int Compare(int x, int y)
            {
                return getAngle(x) > getAngle(y) ? 1 : -1;
            }
        }

        private class referenceComparer : Comparer<int>
        {
            private double[] refArray;

            public referenceComparer(double[] refArray)
            {
                this.refArray = refArray;
            }

            public override int Compare(int x, int y)
            {
                if (refArray[x] > refArray[y])
                    return 1;
                if (refArray[x] < refArray[y])
                    return -1;
                return 0;
            }
        }
    }


}
