﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TSP
{
    class Edge : IComparable<Edge>
    {
        public int From { get; private set; }
        public int To { get; private set; }
        public double Weight { get; private set; }
        private double a, b;
        public TSPInput inp { get; private set; }

        public override bool Equals(object obj)
        {
            if (!(obj is Edge))
                return false;
            Edge e = (Edge)obj;
            return e.From == this.From && e.To == this.To;
        }

        public override int GetHashCode()
        {
            return From * 50000 + To;
        }

        public static bool IsCrossing(TSPPoint A, TSPPoint B, TSPPoint C, TSPPoint D)
        {
            double p = (B.y - A.y) / (B.x - A.x),
                q = A.y - p * A.x;
            return IsCrossing(A, B, C, D, p, q);
        }

        public static bool IsCrossing(TSPPoint A, TSPPoint B, TSPPoint C, TSPPoint D, double p, double q)
        {
            if (B.ID == C.ID || A.ID == C.ID || A.ID == D.ID || B.ID == D.ID)
                return false;
            double res1 = p * C.x + q - C.y,
                res2 = p * D.x + q - D.y;
            if ((res1 < 0 && res2 > 0) || (res1 > 0 && res2 < 0))
                return true;
            return false;
            //return (res1 < 0 && res2 > 0) || (res1 > 0 && res2 < 0);
        }

        public bool IntersectsWith(Edge other)
        {
            //return isCrossing(inp.getPoint(from), inp.getPoint(to), other.inp.getPoint(other.from), other.inp.getPoint(other.to));

            return (a * inp.GetPoint(other.From).x + b < inp.GetPoint(other.From).y ^
                a * inp.GetPoint(other.To).x + b < inp.GetPoint(other.To).y) &&
            (other.a * inp.GetPoint(this.From).x + other.b < inp.GetPoint(this.From).y ^
                other.a * inp.GetPoint(this.To).x + other.b < inp.GetPoint(this.To).y);
        }

        public Edge(int from, int to, TSPInput inp)
        {
            this.From = from;
            this.To = to;
            this.Weight = inp.GetDistance(from, to);
            this.inp = inp;
            this.a = (inp.GetPoint(to).y - inp.GetPoint(from).y) / (inp.GetPoint(to).x - inp.GetPoint(from).x);
            this.b = inp.GetPoint(to).y - inp.GetPoint(to).x * a;
        }

        public Edge(int from, int to, TSPInput inp, double weight)
        {
            this.From = from;
            this.To = to;
            this.Weight = weight;
            this.inp = inp;
            this.a = (inp.GetPoint(to).y - inp.GetPoint(from).y) / (inp.GetPoint(to).x - inp.GetPoint(from).x);
            this.b = inp.GetPoint(to).y - inp.GetPoint(to).x * a;
        }


        #region IComparable<Edge> Members

        public int CompareTo(Edge other)
        {
            if (other.Weight > this.Weight)
                return 1;
            if (other.Weight < this.Weight)
                return -1;
            return 0;
        }

        #endregion
    }
}
