﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace TSP
{
    partial class FitnessForm : Form
    {
        public PictureBox treePictureBox, fitnessPictureBox;

        public TreeNode root;
        public TSPInput inp;
        public TreeEvaluator evaluator = new TreeEvaluator();
        public static Random r = new Random();

        public int treeDepth, leafsCount;
        public double minVal, maxVal;

        public void ComputeLeafs()
        {
            this.leafsCount = root.ComputeLeafs();
        }

        public static TreeNode BuildTree(TSPInput inp, List<int> permutation = null)
        {
            TreeNode root = new TreeNode(inp);
            root.site = 0;
            if (permutation == null)
                root.CreateSuccessors(new HashSet<int>() { 0 }, -1);
            else
                root.CreateSuccessors(new HashSet<int>() { 0 }, -1, permutation);
            return root;
        }

        public void BuildTree(List<int> permutation = null)
        {
            root = FitnessForm.BuildTree(inp, permutation);
            this.leafsCount = root.ComputeLeafs();
            minVal = root.GetMinimum();
            maxVal = root.GetMaximum();
            treeDepth = root.GetDepth();
        }

        public FitnessForm(TSPInput inp)
        {
            this.setComponents();
            this.inp = inp;
        }

        public FitnessForm()
        {
            this.setComponents();
        }

        private void setComponents()
        {
            InitializeComponent();
            this.treePictureBox = TreePictureBox;
            this.fitnessPictureBox = FitnessPictureBox;
        }

        private void searchPermutations(List<int> available, List<int> permutation)
        {

        }

        private void evalButton_Click(object sender, EventArgs e)
        {
            valueLabel.Text = evaluator.Eval(this.root).ToString();
        }

        private void showPermutation(List<int> permutation)
        {
            BuildTree(permutation);
            ComputeLeafs();
            //Length_label.Text = treeS.leafsCount.ToString();
            TreeVisualizer tviz = new TreeVisualizer(this);
            tviz.Draw(root, maxVal, minVal);
            valueLabel.Text = evaluator.Eval(this.root).ToString();
        }

        private void RebuildWithPermutationbutton_Click(object sender, EventArgs e)
        {
            showPermutation(CreateRandomPermutation(inp.NodesCount));
        }

        private void topology_button_Click(object sender, EventArgs e)
        {
            new FitnessLandscapeTopology.Form1(this.inp.NodesCount, q => PermutationStandardFactory.GetInstance().Create(q.ToArray(), inp).Evaluate()).ShowDialog();
        }

        public static void ChangeToNextPermutation(List<int> permutation)
        {
            HashSet<int> available = new HashSet<int>();
            int i = permutation.Count - 1;
            //while (i > 0 && (permutation[i] < permutation[i - 1] || !available.Contains TODO - nesmi obsahovat ani nic vetsiho (permutation[i] + 1)) )
            {
                available.Add(permutation[i]);
                i--;
            }
            if (i == 0 && permutation[0] == permutation.Count - 1)
            {
                permutation = null;
                return;
            }
            available.Add(permutation[i]);
            permutation[i]++;
            available.Remove(permutation[i]);

            var f = available.ToList();
            f.Sort();

            foreach (var item in f)
            {
                i++;
                permutation[i] = item;
            }
        }

        public static List<int> CreateRandomPermutation(int count)
        {
            List<int> result = new List<int>(count);
            List<int> notUsed = new List<int>();
            for (int i = 1; i < count; i++)
                notUsed.Add(i);

            while (notUsed.Count > 0)
            {
                int index = r.Next(notUsed.Count);
                result.Add(notUsed[index]);
                notUsed.RemoveAt(index);
            }
            //result.OrderBy(i => r.Next(2 * count));
            //result.Sort((x, y) => r.Next(2 * count));
            return result;
        }

        private void OptimalSolve_button_Click(object sender, EventArgs e)
        {
            TSPPreprocessor p = new TSPPreprocessor();
            p.FindBestPermutation(this.inp);
            showPermutation(p.bestPermutation);
        }
    }

}
