﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace TSP
{
    public partial class Form1 : Form
    {
        TSPInput inp;
        static TSPVisualizer vis;
        TSPSolver solver;
        TSPSolution solution;
        FitnessForm treeS;
        Random r = new Random();    //set random seed here if needed

        public Form1()
        {
            InitializeComponent();
            vis = new TSPVisualizer(pictureBox1);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            inp = TSPGenerator.GenerateUniform(int.Parse(NodesCount_textBox.Text), r);
            vis.Draw(inp);
        }

        public void SolveAndShow()
        {
            solution = solver.Solve(inp);
            solution.ComputeDistance();
            Length_label.Text = solution.TotalDistance.ToString();
            vis.Draw(solution);
        }

        private void generateClusters_button_Click(object sender, EventArgs e)
        {
            inp = TSPGenerator.GenerateClusters(int.Parse(NodesCount_textBox.Text), int.Parse(clusters_textBox.Text), r);
            vis.Draw(inp);
        }

        private void generateHybrid_button_Click(object sender, EventArgs e)
        {
            inp = TSPGenerator.GenerateHybrid(int.Parse(NodesCount_textBox.Text), int.Parse(clusters_textBox.Text), r);
            vis.Draw(inp);
        }

        private void ClustersAntiGreedy_button_Click(object sender, EventArgs e)
        {
            inp = TSPGenerator.GenerateClustersAntiGreedy(int.Parse(NodesCount_textBox.Text), int.Parse(clusters_textBox.Text), r);
            vis.Draw(inp);
        }

        private void CreateTreeButton_Click(object sender, EventArgs e)
        {
            if (treeS == null || treeS.IsDisposed)
                treeS = new FitnessForm(inp);
            treeS.inp = inp;
            treeS.Show();
            treeS.BuildTree();
            treeS.ComputeLeafs();
            Length_label.Text = treeS.leafsCount.ToString();
            TreeVisualizer tviz = new TreeVisualizer(treeS);
            tviz.Draw(treeS.root, treeS.maxVal, treeS.minVal);
        }

        private void ShowNumberscheckBox_CheckedChanged(object sender, EventArgs e)
        {
            vis.SetShowNumbers(ShowNumberscheckBox.Checked);
        }

        private void Extern_button_Click(object sender, EventArgs e)
        {
            Process p = new Process();
            p.StartInfo.FileName = @".\solver.exe";
            p.StartInfo.UseShellExecute = false;
            p.StartInfo.RedirectStandardInput = true;
            p.StartInfo.RedirectStandardOutput = true;
            p.Start();
            p.StandardInput.WriteLine(inp.NodesCount);
            for (int i = 0; i < inp.NodesCount; i++)
			{
			    p.StandardInput.WriteLine((int)(Math.Floor(inp.GetPoint(i).x)) + " " + (int)(Math.Floor(inp.GetPoint(i).y)));
			}
            string result = p.StandardOutput.ReadLine();
            solution = TSPSolution.fromString(result, inp);
            solution.ComputeDistance();
            Length_label.Text = solution.TotalDistance.ToString();
            vis.Draw(solution);
        }

        private void Solve_button_Click(object sender, EventArgs e)
        {
            if (engines_listBox.SelectedIndex == 3)
            {
                TreeAproximationSolver s = new TreeApproximativeSolverFactory().Create();
                solution = s.Solve(inp);
                solution.ComputeDistance();
                Length_label.Text = solution.TotalDistance.ToString();
                vis.DrawSpanningTree(inp, s.successorsList);
                vis.Draw(solution, false);
            }
            else
            {
                switch (engines_listBox.SelectedIndex)
                {
                    case 0:
                        solver = new RandomSolverFactory(this.r).Create();
                        break;
                    case 1:
                        solver = new GreedySolverFactory().Create();
                        break;
                    case 2:
                        solver = new GreedyGrowingSolverSolverFactory().Create();
                        break;
                    case 4:
                        solver = new OptimalSolverFactory().Create();
                        break;
                    case 5:
                        solver = new CentroidBubbleSolver();
                        break;
                    case 6:
                        /*
                        solver = new PlannerWrapperFactory().create(plannerType.AstarBlind);
                        break;
                         */
                        HillClimbingTSPManager s = new HillClimbingTSPManager(vis, r);
                        solution = s.Solve(inp);
                        solution.ComputeDistance();
                        Length_label.Text = solution.TotalDistance.ToString();
                        vis.Draw(solution, false);
                        return;
					case 7:
                        /*
						TSPSolver so = new GASolver(vis, r);
						solution = so.Solve(inp);
						solution.ComputeDistance();
						Length_label.Text = solution.TotalDistance.ToString();
						vis.Draw(solution);
                        */
						return;
					
				}
                SolveAndShow();
            }
        }

        private void Load_button_Click(object sender, EventArgs e)
        {
            LoadInput_openFileDialog.ShowDialog();
        }

        private void Save_button_Click(object sender, EventArgs e)
        {
            SaveInput_saveFileDialog.ShowDialog();
        }

        private void LoadInput_openFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            inp = TSPInput.Load(LoadInput_openFileDialog.FileName);
            vis.Draw(inp);
        }

        private void SaveInput_saveFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            inp.Save(SaveInput_saveFileDialog.FileName);
        }

    }
}
