﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TSP
{
    class RandomSolver : TSPSolver
    {
        private List<int> available;
        private Random r;

        public RandomSolver(Random r)
        {
            this.r = r;
            available = new List<int>();
        }

        #region TSPSolver Members

        public TSPSolution Solve(TSPInput input)
        {
            available.Clear();
            for (int i = 1; i < input.NodesCount; i++)
                available.Add(i);
            TSPSolution result = new TSPSolution(input);
            int j = 0;
            for (int i = 0; i < input.NodesCount - 1; i++)
            {
                int randomIndex = r.Next(available.Count);
                result.SetSuccessor(j, available[randomIndex]);
                j = available[randomIndex];
                available.RemoveAt(randomIndex);
            }
            result.SetSuccessor(j, 0);
            return result;
        }

        #endregion
    }

    class RandomSolverFactory : TSPSolverFactory<RandomSolver>
    {
        Random r;

        public RandomSolverFactory(Random r)
        {
            this.r = r;
        }

        #region TSPSolverFactory<RandomSolver> Members

        public RandomSolver Create()
        {
            return new RandomSolver(r);
        }

        #endregion
    }
}
